﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using C4F.VistaP2P.Common;
using C4F.VistaP2P.Win.Chat;
using C4F.VistaP2P.Win.File;
using C4F.VistaP2P.Win.Picture;
using C4F.VistaP2P.Win.Media;
using C4F.VistaP2P.Win;
namespace C4F.VistaP2P.Win.NetworkADV
{

    /// <summary>
    /// This NetworkManager or the component NetworkManagerADV is required to use
    /// any of the Peer 2 Peer Controls that are part of Coding 4 Fun.
    /// </summary>
    [Description("This component or the control <NetworkManager> is necessary to use any of the C4F P2P Controls." +
                 "The only difference between the two is that this one is a component and the other is a control"),
      ToolboxBitmap(typeof(resfinder), "C4F_P2PWinformControls.c4f.bmp"),
    Designer(typeof(NetworkDesignerADV))]            
    public partial class NetworkManagerADV : Component
    {
        public NetworkManagerADV(IContainer container)
        {
            container.Add(this);

            InitializeComponent();

            if (UserName == null || NetworkName == null || Password == null || NetworkName == null || LeaveNetwork == null)
            {
                MessageBox.Show("Please assign a textbox control to the UserName, NetworkName, and Password. This can be done in the designer properties of the component (click on the NetworkManagerADV and press F4)");
            }
            else if (Status == null)
            {
                MessageBox.Show("Please assign a lable to the property Status. This can be done in the designer properties of the component (click on the NetworkManagerADV and press F4)");
            }
            else if (LeaveNetwork == null || JoinNetwork == null)
            {
                MessageBox.Show("Please assign a buttons to the properties JoinNetwork and LeaveNetwork. This can be done in the designer properties of the component (click on the NetworkManagerADV and press F4)");
            }

        }

        /// <summary>
        /// The PeerChannel class is put in a different thread than the UI so the UI remains responsive. 
        /// </summary>
        private BackgroundWorker mPeerChannelThread;

        /// <summary>
        /// the lower level Peer2PeerLibrary
        /// </summary>
        private P2PLib mP2PLib;

        /// <summary>
        /// Used for streaming audio through a local url
        /// </summary>
        private StreamingHttpListener mAudioHttpListener;
        
        /// <summary>
        /// Used for streaming video through a local url
        /// </summary>
        private StreamingHttpListener mVideoHttpListener;

        /// <summary>
        /// The current state of the network.
        /// </summary>
        private SystemState mState = SystemState.LoggedOut;

        private PictureControl mPicture = null;
        /// <summary>
        /// gets or sets the picture control to use. ALso registers and ungregisters for the appropriate events 
        /// </summary>
        /// <value>the picture control</value>
        [

           Description("Set the Picture Control to use with this NetworkManager"),
           Category("P2P"),
        ]
        public PictureControl Picture
        {
            get { return mPicture; }
            set
            {
                if (mPicture != null)
                {
                    mPicture.SendingPicture -= new EventHandler<PictureChangedEventArgs>(mPicture_SendingPicture);
                }

                mPicture = value;
                if (mPicture != null)
                {
                    mPicture.SetConnectionStatus(mState);
                    mPicture.SendingPicture += new EventHandler<PictureChangedEventArgs>(mPicture_SendingPicture);
                }
            }
        }

        /// <summary>
        /// This function listens for the SendingPicture event.
        /// </summary>
        /// <param name="sender">unused</param>
        /// <param name="e">the picture data</param>
        void mPicture_SendingPicture(object sender, PictureChangedEventArgs e)
        {
            mP2PLib.SharePicture(e.Packet);
        }


        /// <summary>
        /// a reference to the chat control
        /// </summary>
        private ChatControl mChat = null;

        /// <summary>
        /// gets or sets the chat control to use
        /// </summary>
        /// <value>a chat control</value>
        [

           Description("Set the Chat Control to use with this NetworkManager"),           
           Category("P2P"),          
        ]
        public ChatControl Chat
        {
            get { return mChat; }
            set
            {
                if (mChat != null)
                {
                    mChat.SendingText -= new EventHandler<ChatChangedEventArgs>(mChat_SendingText);
                }

                mChat = value;
                if (mChat != null)
                {
                    mChat.SetConnectionStatus(mState);
                    mChat.SendingText += new EventHandler<ChatChangedEventArgs>(mChat_SendingText);
                }
            }
        }

        /// <summary>
        /// listens for the sending TextEvent from the chat control
        /// </summary>
        /// <param name="sender">unused</param>
        /// <param name="e">the message sent</param>
        void mChat_SendingText(object sender, ChatChangedEventArgs e)
        {
            mP2PLib.TextPeerChannelHelper.SendTextMessage(UserName.Text, e.Message);
        }


        /// <summary>
        /// a reference to the file control
        /// </summary>
        private FileControl mFile = null;

        /// <summary>
        /// the file control property
        /// </summary>
        /// <value>a file control</value>
        [

           Description("Set the File Control to use with this NetworkManager"),
           Category("P2P"),
        ]
        public FileControl File
        {
            get { return mFile; }
            set
            {
                if (mFile != null)
                {
                    mFile.SendingFile -= new EventHandler<FileChangedEventArgs>(mFile_SendingFile);
                }

                mFile = value;
                if (mFile != null)
                {
                    mFile.SetConnectionStatus(mState);
                    mFile.SendingFile += new EventHandler<FileChangedEventArgs>(mFile_SendingFile);
                }
            }
        }

        /// <summary>
        /// This function listens for the SendingFile event from the file control
        /// </summary>
        /// <param name="sender">unused</param>
        /// <param name="e">the file data</param>
        void mFile_SendingFile(object sender, FileChangedEventArgs e)
        {
            e.Packet.fileName = UserName.Text;
            mP2PLib.ShareFile(e.Packet);
        }


        /// <summary>
        /// a reference to a mediaControl to use as the audio source.
        /// </summary>
        private MediaControl mAudioControl = null;

        /// <summary>
        /// The media control to use with this NetworkManager to play audio.
        /// </summary>
        [
           Description("Set the Audio Control to use with this NetworkManager"),
           Category("P2P"),
        ]
        public MediaControl Audio
        {
            get { return mAudioControl; }
            set
            {
                if (mAudioControl != null)
                {
                    mAudioControl.SendingStream -= new EventHandler<SendStreamEventArgs>(Audio_SendingStream);
                    mAudioControl.CancelStream -= new EventHandler(Audio_CancelStream);
                }

                mAudioControl = value;
                if (mAudioControl != null)
                {
                    mAudioControl.lblThirdLineLabel.Visible = true;
                    mAudioControl.ThirdLineValue.Visible = true;
                    //mMediaControl.SetConnectionStatus(mState);
                    mAudioControl.SendingStream += new EventHandler<SendStreamEventArgs>(Audio_SendingStream);
                    mAudioControl.CancelStream += new EventHandler(Audio_CancelStream);
                }
            }
        }

        /// <summary>
        /// Relays the event containing the picture be sent to the lower level P2PLibrary
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Audio_SendingStream(object sender, SendStreamEventArgs e)
        {
            if (null != mP2PLib)
            {
                mAudioHttpListener.SendStreamingData(e.FullFileName);

            }
        }

        /// <summary>
        /// Relays the event containing the picture be sent to the lower level P2PLibrary
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Video_SendingStream(object sender, SendStreamEventArgs e)
        {
            if (null != mP2PLib)
            {
                mVideoHttpListener.SendStreamingData(e.FullFileName);
            }
        }

        /// <summary>
        /// Relays the event containing the picture be sent to the lower level P2PLibrary
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Audio_CancelStream(object sender, EventArgs e)
        {
            if (null != mP2PLib)
            {
                mP2PLib.StreamedAudio.CancelSendingStream();
            }
        }

        /// <summary>
        /// Relays the event containing the picture be sent to the lower level P2PLibrary
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Video_CancelStream(object sender, EventArgs e)
        {
            if (null != mP2PLib)
            {
                mP2PLib.StreamedVideo.CancelSendingStream();
            }
        }
        //A reference to the Picture Control if one is assigned.
        private MediaControl mVideoControl = null;

        /// <summary>
        /// The Picture control to use with this NetworkManager.
        /// </summary>
        [
           Description("Set the Video Control to use with this NetworkManager"),
           Category("P2P"),
        ]
        public MediaControl Video
        {
            get { return mVideoControl; }
            set
            {
                if (mVideoControl != null)
                {
                    mVideoControl.SendingStream -= new EventHandler<SendStreamEventArgs>(Video_SendingStream);
                    mVideoControl.CancelStream -= new EventHandler(Video_CancelStream);
                }

                mVideoControl = value;
                if (mVideoControl != null)
                {
                    mVideoControl.lblThirdLineLabel.Visible = false;
                    mVideoControl.ThirdLineValue.Visible = false;
                    //mMediaControl.SetConnectionStatus(mState);
                    mVideoControl.SendingStream += new EventHandler<SendStreamEventArgs>(Video_SendingStream);
                    mVideoControl.CancelStream += new EventHandler(Video_CancelStream);
                }
            }
        }



        /// <summary>
        /// a refernce to the UserName control if set.
        /// </summary>
        private TextBox mUserName = null;
        
        /// <summary>
        /// Gets or sets the Username control
        /// </summary>
        /// <value>the username textbox control</value>
        [
           Description("Specify a textbox to use as the username name, otherwise it will use the networkManager's networkname textbox"),
           Category("P2P/controls"),           
        ]
        public TextBox UserName
        {
            get 
            {              
                return mUserName; 
            }
            set
            {
                if (value != null)
                {
                    mUserName = value;
                }
            }
        }

        /// <summary>
        /// a reference to the Password control if set.
        /// </summary>
        private TextBox mPassword = null;

        /// <summary>
        /// Gets or sets the Password control to use
        /// </summary>
        /// <value>the textbox control to use as a password</value>
        [
           Description("Specify a textbox to use as the password textbox, otherwise it will use the networkManager's passwordName textbox"),
           Category("P2P/controls")        
        ]
        public TextBox Password
        {
            get 
            {                
                return mPassword;            
            }
            set
            {
                mPassword = value;
            }
        }

        /// <summary>
        /// a reference to the network name.
        /// </summary>
        private TextBox mNetworkName = null;
        
        /// <summary>
        /// The textbox to set or get to use for the network name.
        /// </summary>
        /// <value></value>
        [
           Description("Specify a textbox to use as the networkName textbox, otherwise it will use the networkManager's networkName textbox"),
           Category("P2P/controls")           
        ]
        public TextBox NetworkName
        {
            get 
            {                
                return mNetworkName;               
            }
            set
            {
                mNetworkName = value;
            }
        }


        /// <summary>
        /// a reference to the status label
        /// </summary>
        private Label mStatus = null;

        /// <summary>
        /// Gets or sets the label to use as the status of the network connection.
        /// </summary>
        /// <value>the label to set or get</value>
        [
           Description("Specify a label to use as the networkManagers status, otherwise it will use the networkManager's status label"),
           Category("P2P/controls")
        ]
        public Label Status
        {
            get
            {
                return mStatus;
            }
            set
            {
                mStatus = value;
            }
        }

        /// <summary>
        /// A reference to the button 
        /// </summary>
        private Button mJoinNetwork = null;
        
        /// <summary>
        /// Gets or sets the button to use as the join network button
        /// </summary>
        /// <value></value>
        [
           Description("Specify a button to use as the join button, otherwise it will use the networkManager's join button"),
           Category("P2P/controls")
        ]
        public Button JoinNetwork
        {
            get 
            {
                return mJoinNetwork;
            }
            set
            {
                if (mJoinNetwork != null)
                {
                    mJoinNetwork.Click -= new EventHandler(JoinNetwork_Click);
                }

                mJoinNetwork = value;
                if (value != null)
                {
                    mJoinNetwork.Click += new EventHandler(JoinNetwork_Click);
                }
            }
        }

        /// <summary>
        /// a reference to the leave button network.
        /// </summary>
        private Button mLeaveNetwork = null;
        
        /// <summary>
        /// Gets or sets the Leave button to use.
        /// </summary>
        /// <value></value>
        [
           Description("Specify a button to use as the leave button, otherwise it will use the networkManager's leave button"),
           Category("P2P/controls")           
        ]
        public Button LeaveNetwork
        {
            get 
            {
                return mLeaveNetwork; 
            }
            set
            {
                if (mLeaveNetwork != null)
                {
                    mLeaveNetwork.Click -= new EventHandler(LeaveNetwork_Click);
                }


                if (value != null)
                {
                   
                    mLeaveNetwork = value;
                    mLeaveNetwork.Enabled = false;
                    mLeaveNetwork.Click += new EventHandler(LeaveNetwork_Click);
                }
            }
        }



        /// <summary>
        /// The constructor that just initializes the forms.
        /// </summary>
        public NetworkManagerADV()
        {
            InitializeComponent();
        }

        /// <summary>
        /// causes the network to shut down.
        /// </summary>
        /// <param name="sender">unused.</param>
        /// <param name="e">unused</param>
        void LeaveNetwork_Click(object sender, EventArgs e)
        {
            End();
        }

        /// <summary>
        /// listens for the join button to be hit.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void JoinNetwork_Click(object sender, EventArgs e)
        {
            Start();
        }        

        
       


        /// <summary>
        /// This is the main function that takes the input from the textboxes in the window.
        /// It launches all of the threads necessary to stream audio and also starts the peerChannel thread.  This thread
        /// starts the PeerChannelWrapper class and opens a connection to the mesh.
        /// </summary>
        protected void Start()
        {

            if (UserName == null || NetworkName == null || Password == null || NetworkName == null || LeaveNetwork == null)
            {
                MessageBox.Show("Please assign a textbox control to the UserName, NetworkName, and Password");
            }
            else if (Status == null)
            {
                MessageBox.Show("Please assign a lable to the property Status");
            }
            else if (LeaveNetwork == null || JoinNetwork == null)
            {
                MessageBox.Show("Please assign a buttons to the properties JoinNetwork and LeaveNetwork");
            }
            else
            {

                //validate input and limit to alphanumeric chars
                Regex regex = new Regex("[a-zA-Z0-9]");
                if (!string.IsNullOrEmpty(UserName.Text) && !string.IsNullOrEmpty(NetworkName.Text) &&
                    regex.IsMatch(UserName.Text) && regex.IsMatch(NetworkName.Text))
                {

                    UserName.Enabled = false;
                    NetworkName.Enabled = false;
                    Password.Enabled = false;
                    JoinNetwork.Enabled = false;
                    LeaveNetwork.Enabled = true;

                    //check initial conditions..
                    if (!(null == mP2PLib && mPeerChannelThread == null))
                    {
                        //error unable to start something didn't get cleaned up correctly when we left the network.
                        MessageBox.Show("An Error occurred when trying to initialize the Network. Please restart the application");
                        return;
                    }

                    //update the state.
                    mState = SystemState.LoggingIn;
                    Status.Text = "Status: Connecting...";
                    //start up the peerchannel thread which will actually do the work
                    mPeerChannelThread = new System.ComponentModel.BackgroundWorker();
                    mPeerChannelThread.DoWork += new System.ComponentModel.DoWorkEventHandler(mP2PLibThread);
                    mPeerChannelThread.WorkerReportsProgress = true;
                    mPeerChannelThread.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(P2PLibThreadCompleted);
                    mPeerChannelThread.WorkerSupportsCancellation = true;
                    mPeerChannelThread.RunWorkerAsync();

                }
                else
                {
                    MessageBox.Show("Please Enter a UserName and NetworkName that contains only alpha-numeric characters");
                }
            }
        }


        
        /// <summary>
        /// This function listens for the network status to change and notifies the attached controls
        /// </summary>
        /// <param name="sender">ignored</param>
        /// <param name="e">information about how the network changed</param>
        protected void P2PLib_StatusChanged(object sender, C4F.VistaP2P.Common.StatusChangedEventArgs e)
        {
            if (Audio != null)
            {
                Audio.SetStatusChanged(UserName.Text, e);
            }

            if (Chat != null)
            {
                Chat.SetStatusChanged(UserName.Text, e);
            }
            if (Picture != null)
            {
                Picture.SetStatusChanged(UserName.Text, e);
            }
            if (File != null)
            {
                File.SetStatusChanged(UserName.Text, e);
            }

            if (Video != null)
            {
                Video.SetStatusChanged(UserName.Text, e);
            }


            if (e.Member == UserName.Text)
            {
                if (e.NewNodeJoined)
                {
                   Utility.SetTextLabel(Status,"Status: Connected...");
                    mState = SystemState.LoggedIn;
                }

                if (e.NodeLeft)
                {
                    Utility.SetTextLabel(Status,"Status: Disconnected...");
                    mState = SystemState.LoggedOut;            
                }
            }
            else
            {
                if (e.NewNodeJoined)
                {
                    Chat_ChatChanged(this, new ChatChangedEventArgs( string.Format("{0} just joined!\n", e.Member)));
                }

                if (e.NodeLeft)
                {
                    Chat_ChatChanged(this, new ChatChangedEventArgs( string.Format("{0} just left!\n", e.Member)));
                }
            }                       
        }

        /// <summary>
        /// listens for a chat message to arrive and relays it to the chat control.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Chat_ChatChanged(object sender, ChatChangedEventArgs e)
        {
            if (Chat != null)
            {
              Chat.SetMessageReceived( e);
            }
        }

        /// <summary>
        /// This is the main loop for the peerChannelThread.
        /// It creates a new instance of the PeerChannelWrapper, which opens the mesh.
        /// </summary>
        /// <param name="sender">unused</param>
        /// <param name="doWorkArgs">unused</param>
        private void mP2PLibThread(object sender, DoWorkEventArgs doworkargs)
        {
            mP2PLib = new P2PLib();
            if (false == mP2PLib.PeerChannelWrapperStart(UserName.Text, NetworkName.Text, Password.Text))
            {
                //something went wrong opening the connection...bail out 
                End();
            }

            //register for the events from the p2p library that we are interested in
            mP2PLib.StatusChanged += new EventHandler<StatusChangedEventArgs>(P2PLib_StatusChanged);
            mP2PLib.TextPeerChannelHelper.ChatChanged += new EventHandler<ChatChangedEventArgs>(Chat_ChatChanged);
            mP2PLib.Picture.PictureChanged += new EventHandler<PictureChangedEventArgs>(Picture_PictureChanged);
            mP2PLib.FilePeerChannelHelper.FileChanged += new EventHandler<FileChangedEventArgs>(FilePeerChannelHelper_FileChanged);


            if (mAudioControl != null)
            {
                mP2PLib.StreamedAudio.StreamChanged += new EventHandler<StreamedChangedEventArgs>(mP2P_StreamedAudioListener);
                ////initialize the audio parameters.
                mAudioHttpListener = new StreamingHttpListener(UserName.Text + "/audio/",
                    AudioBackgroundWorkerThreadFinished, AudioStatusMessage, mP2PLib.StreamedAudio.StartStream,
                    mP2PLib.StreamedAudio.SendStream);
            }


            if (mVideoControl != null)
            {
                mP2PLib.StreamedVideo.StreamChanged += new EventHandler<StreamedChangedEventArgs>(mP2P_StreamedVideoListener);

                mVideoHttpListener = new StreamingHttpListener(UserName.Text + "/video/",
                     VideoBackgroundWorkerThreadFinished, VideoStatusMessage, mP2PLib.StreamedVideo.StartStream,
                     mP2PLib.StreamedVideo.SendStream);
            }

            
            
            //just sit and wait until the main window tells us to quit.
            //this leaves the peerchannelwrapper instance alive and running in this thread so it can process commands.
            while (true)
            {
                System.Threading.Thread.Sleep(1000);
                if (mPeerChannelThread.CancellationPending)
                {
                    //someone told us to quit.
                    mPeerChannelThread = null;
                    break;
                }
            }
        }


        /// <summary>
        /// listens for the status of the video thread to change and notifies the video control if attached
        /// </summary>
        /// <param name="msg">the message to send to the video control</param>
        public void VideoStatusMessage(string msg)
        {
            if (mVideoControl != null)
            {
                mVideoControl.SetStatusMessage(msg);
            }
        }

        /// <summary>
        /// listens for the status of the audio thread to change and notifies the video control if attached
        /// </summary>
        /// <param name="msg">the message to send to the audio control</param>
        public void AudioStatusMessage(string msg)
        {
            if (mAudioControl != null)
            {
                mAudioControl.SetStatusMessage(msg);
            }
        }

        
        /// <summary>
        /// delegate to make sure we change the video control on the video controls thread. Otherwise an exception will be thrown.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        delegate void mP2P_StreamedVideoHandler(object sender, StreamedChangedEventArgs e);

        /// <summary>
        /// This function gets called when we receive a video message. If it is the first packet in the audio stream
        /// we use a delegate and have the UI(user interface) thread call this function.  When it does CheckAccess will be true
        /// and the controls will be updated.  
        /// </summary>
        /// <param name="e"></param>
        public void mP2P_StreamedVideoListener(object sender, StreamedChangedEventArgs e)
        {
            if (!mVideoControl.InvokeRequired)
            {
                bool play = false;
                play = e.StreamedPacket.senderNodeName == UserName.Text;


                DialogResult mbr = DialogResult.Cancel;
                if (false == play)
                {
                    mbr = MessageBox.Show(e.StreamedPacket.senderNodeName + " just sent you a Video. Select OK to accept it, otherwise select Cancel", "A file was sent to you!", MessageBoxButtons.OKCancel);
                }

                if (play || mbr == DialogResult.OK)
                {
                    Utility.SetTextLabel(mVideoControl.FirstLineLabel, "Title");
                    Utility.SetTextLabel(mVideoControl.SecondLineLabel, "File");
                    Utility.SetTextLabel(mVideoControl.FirstLineValue, e.StreamedPacket.title);
                    Utility.SetTextLabel(mVideoControl.SecondLineValue, e.StreamedPacket.fileName);

                    mVideoControl.StartPlay("http://localhost:8088/" + UserName.Text + "/video/" + StreamingHttpListener.SafeHttpString(e.StreamedPacket.fileName));
                }
            }
            else
            {
                mVideoHttpListener.StreamedChanged(e);
                if (e.StreamedPacket.packetNumber == 0)
                {
                    mVideoControl.Invoke(new mP2P_StreamedVideoHandler(mP2P_StreamedVideoListener), new object[] { sender, e });
                }
            }
        }


        /// <summary>
        /// This gets called when the audio thread has finished. If the video thread and the peerchannel thread have all quit,
        /// then it sets the state back to initial and allows the user to join another network.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void VideoBackgroundWorkerThreadFinished(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(String.Format("An Error occurred in the Video Processing Thread:{0}", e.Error.Message));
            }
            else
            {
                mVideoHttpListener = null;
                if (mPeerChannelThread == null && mAudioHttpListener == null)
                {
                    Utility.SetTextLabel(Status,"Status: Disconnected...");
                    mState = SystemState.LoggedOut;
                    UserName.Enabled = true;
                    NetworkName.Enabled = true;
                    Password.Enabled = true;
                    JoinNetwork.Enabled = true;
                    LeaveNetwork.Enabled = false;
                }
            }
        }


        /// <summary>
        /// This function gets called when we receive a video message. If it is the first packet in the audio stream
        /// we use a delegate and have the UI(user interface) thread call this function.  When it does CheckAccess will be true
        /// and the controls will be updated.  
        /// </summary>
        /// <param name="e"></param>
        delegate void mP2P_StreamedAudioHandler(object sender, StreamedChangedEventArgs e);
        public void mP2P_StreamedAudioListener(object sender, StreamedChangedEventArgs e)
        {


            if (!mAudioControl.InvokeRequired)
            {
                bool play = false;
                play = e.StreamedPacket.senderNodeName == UserName.Text;

                DialogResult mbr = DialogResult.Cancel;
                if (false == play)
                {
                    mbr = MessageBox.Show(e.StreamedPacket.senderNodeName + " just sent you an audio file . Select OK to accept it, otherwise select Cancel", "A file was sent to you!", MessageBoxButtons.OKCancel);
                }

                if (play || mbr == DialogResult.OK)
                {
                    Utility.SetTextLabel(mAudioControl.FirstLineLabel, "Title");
                    Utility.SetTextLabel(mAudioControl.SecondLineLabel, "Artist");
                    Utility.SetTextLabel(mAudioControl.SecondLineLabel, "Album");

                    Utility.SetTextLabel(mAudioControl.FirstLineValue, e.StreamedPacket.title);
                    Utility.SetTextLabel(mAudioControl.SecondLineValue, e.StreamedPacket.artist);
                    Utility.SetTextLabel(mAudioControl.SecondLineValue, e.StreamedPacket.artist);

                    mAudioControl.StartPlay("http://localhost:8088/" + UserName.Text + "/audio/" + StreamingHttpListener.SafeHttpString(e.StreamedPacket.fileName));
                }
            }
            else
            {
                mAudioHttpListener.StreamedChanged(e);
                if (e.StreamedPacket.packetNumber == 0)
                {
                    mAudioControl.Invoke(new mP2P_StreamedAudioHandler(mP2P_StreamedAudioListener), new object[] { sender, e });
                }
            }
        }


        /// <summary>
        /// This gets called when the audio thread has finished. If the video thread and the peerchannel thread have all quit,
        /// then it sets the state back to initial and allows the user to join another network.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void AudioBackgroundWorkerThreadFinished(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(String.Format("An Error occurred in the Audio Processing Thread:{0}", e.Error.Message));
            }
            else
            {
                mAudioHttpListener = null;

                if (mPeerChannelThread == null && mVideoHttpListener == null)
                {
                    Utility.SetTextLabel(Status,"Status: Disconnected...");
                    mState = SystemState.LoggedOut;
                    UserName.Enabled = true;
                    NetworkName.Enabled = true;
                    Password.Enabled = true;
                    JoinNetwork.Enabled = true;
                    LeaveNetwork.Enabled = false;
                }
            }
        }


        void FilePeerChannelHelper_FileChanged(object sender, FileChangedEventArgs e)
        {
            if (File != null)
            {
                File.SetFileReceived(e);
            }
        }

        void Picture_PictureChanged(object sender, PictureChangedEventArgs e)
        {
            if (Picture != null)
            {
                Picture.SetNewPicture(e);
            }
        }



        ///// <summary>
        ///// Before going back to the initial state the P2PThread the following three threads should all quit
        /////  mVideoHttpListener mAudioHttpListener  peerChannelThread
        ///// As each thread completes it checks to see if the others have completed. If it is the last one
        ///// then it sets the state back to initial.
        ///// </summary>
        ///// <param name="sender"></param>
        ///// <param name="e"></param>
        private void P2PLibThreadCompleted(object sender, AsyncCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(String.Format("An Error occurred in the P2P Thread: {0}", e.Error.Message));
            }
            else
            {
                Utility.SetTextLabel(Status,"Status: Disconnected...");     
                mPeerChannelThread = null;
                mState = SystemState.LoggedOut;
                UserName.Enabled = true;
                NetworkName.Enabled = true;
                Password.Enabled = true;
                JoinNetwork.Enabled = true;
                LeaveNetwork.Enabled = false;
            }
        }

        ///// <summary>
        ///// Logs us out of the network and shuts everything down. 
        ///// </summary>
        protected void End()
        {
            if (null != mP2PLib)
            {
                mP2PLib.Close();
                mP2PLib = null;
            }

            if (null != mVideoHttpListener)
                mVideoHttpListener.Stop();

            if (null != mAudioHttpListener)
                mAudioHttpListener.Stop();

            if (null != mVideoHttpListener)
            {
                mVideoHttpListener.Quit();
            }

            if (null != mAudioHttpListener)
            {
                mAudioHttpListener.Quit();
            }


            if (null != mPeerChannelThread)
            {
                mPeerChannelThread.CancelAsync();
            }   
        }
    }
}
